/* daemon.c -- Function to become a UNIX daemon 
 * 
 * Copyright (C) 2004 H Zero Seven
 *
 * Authors: Christian Volmering <risk@h07.org>
 *          Enrico Kern <phantom@h07.org>
 * 
 * This file is part of the aexpl-ng distribution.
 * 
 * The aexpl-ng distribution is free software; you can redistribute it 
 * and/or modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2 of 
 * the License, or (at your option) any later version.
 *
 * The aexpl-ng distribution is distributed in the hope that it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied 
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the aexpl-ng distribution; if not, write to the 
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 * MA 02111-1307 USA
 */

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <signal.h>

#ifdef WITH_DMALLOC
#  include <dmalloc.h>
#endif

/* {{{ detach */
int
detach (int nochdir, int noclose)
{
  pid_t pid;

  if ((pid = fork ()) != 0)
    exit (0);
  else if (pid == -1)
    {
      perror ("Could'n fork");
      return (0);
    }

  if (setsid () == -1)
    {
      perror ("Error creating session");
      return (0);
    }

  if (signal (SIGHUP, SIG_IGN) == SIG_ERR)
    {
      perror ("Couldn't install handler for SIGHUP");
      return (0);
    }

  if ((pid = fork ()) != 0)
    exit (0);
  else if (pid == -1)
    {
      perror ("Couldn't fork");
      return (0);
    }

  if (!nochdir)
    {
      if (chdir ("/") == -1)
	{
	  perror ("Couldn't change working directory to '/'");
	  return (0);
	}
    }

  umask (0);

  if (!noclose)
    {
      int fd;

      if ((fd = open ("/dev/null", O_RDWR)) == -1)
	{
	  perror ("Error opening '/dev/null'");
	  return (0);
	}

      close (STDIN_FILENO);
      close (STDOUT_FILENO);
      close (STDERR_FILENO);

      dup2 (fd, STDIN_FILENO);
      dup2 (fd, STDOUT_FILENO);
      dup2 (fd, STDERR_FILENO);

      close (fd);
    }

  return (1);
}

/* }}} */

/*
 * Local variables:
 * tab-width: 8
 * c-basic-offset: 8
 * End:
 * vim600: noet nonu sw=8 ts=8 fdm=marker
 * vim<600: noet nonu sw=8 ts=8
 */
